
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import features.Parameters;
import features.WidgetFactory;

/*
 * TeeChartDemo.java
 *
 * <p>Copyright: Copyright (c) 2004-2008 by Steema Software SL. All Rights
 * Reserved.</p>
 *
 * <p>Company: Steema Software SL</p>
 */

public class TeeChartSWT {
		
	public TeeChartSWT() {
		display = new Display();
		shell = new Shell(display);
		shell.setText("TeeChart for Java");
			
		Monitor monitor = display.getPrimaryMonitor();
		if (monitor != null) {
			Rectangle dim = monitor.getClientArea();
			shell.setSize(dim.width - (dim.width / 3),
                    dim.height - (dim.height / 3));
		}
		
		initUI();
		//shell.pack();
	}
	
	public void run() {
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		display.dispose();		
	}
	
	private Display display;
	private Shell shell;
	
	private Button leftButton, rightButton;
	private Composite sampleContainer;	
	
	static class HelperTree  {
		
		public static TreeItem getNextSibling(Tree tree, TreeItem item, boolean expandChilds) {
			TreeItem sibling = null;
			TreeItem parent = item.getParentItem();
			int index = -1;
			if (expandChilds & item.getItemCount()>0) {
				sibling = item.getItem(0);
			} else {
				if (parent == null) {
					index = tree.indexOf(item);
					if (index != tree.getItemCount()-1) {
						sibling = tree.getItem(++index);
					}
				} else {
					if (item == parent.getItem(parent.getItemCount()-1)) {
						sibling = getNextSibling(tree, parent, false); 
					} else {
						index = parent.indexOf(item);
						sibling = parent.getItem(++index);
					}
				}
			}
			return sibling;
		}

		public static TreeItem getPreviousSibling(Tree tree, TreeItem item, boolean expandChilds) {
			TreeItem sibling = null;
			TreeItem parent = item.getParentItem();
			int index = -1;
			if (parent == null) {
				index = tree.indexOf(item);
				if (index >= 0 ) {
					sibling = tree.getItem(--index);
					if (expandChilds) {
						sibling = getLastChild(sibling);
					}
				}
			} else {
				if (item == parent.getItem(0)) {
					sibling = getPreviousSibling(tree, parent, false); 
				} else {
					index = parent.indexOf(item);
					sibling = parent.getItem(--index);
					if (expandChilds) {
						sibling = getLastChild(sibling);
					}
				}
			}
			return sibling;
		}
		
		public static TreeItem getLastChild(TreeItem item) {
			int childs=item.getItemCount();
			if (childs>0) {
				item = getLastChild(item.getItem(childs-1));
			} 
			return item;
		}
		
	}
	
	class Close extends SelectionAdapter {
		public void widgetSelected(SelectionEvent se) {
			shell.close();
		}
	};
	
	class About extends SelectionAdapter {
		public void widgetSelected(SelectionEvent se) {
			com.steema.teechart.editors.AboutDialog dialog = 
				   new com.steema.teechart.editors.AboutDialog(shell);
			dialog.open();
		}
	};  
	
	class Navigate extends SelectionAdapter {
		public void widgetSelected(SelectionEvent se) {
			if (activeTree.getSelectionCount()>0) {
				TreeItem selection = activeTree.getSelection()[0];				
				TreeItem sibling = null;
				if (se.widget==leftButton) {				
					sibling = HelperTree.getPreviousSibling(activeTree, selection, true);
				} else if (se.widget==rightButton) {
					sibling = HelperTree.getNextSibling(activeTree, selection, true);
				}
				if (sibling != null) {
					activeTree.setSelection(sibling);
				}				
				activeTree.getSelection()[0].setExpanded(true);
				showSamplePane((TeeFeature)activeTree.getSelection()[0].getData());
			}
		}
	};  
	   
	private Label statusLabel;
	
    private Composite createBottomUI(Composite parent) {
    	Composite pane = new Composite(parent, SWT.BORDER);
    	pane.setLayout(new FillLayout());    	
    	statusLabel = new Label(pane, SWT.CENTER);    	
    	return pane;
    }

	private Composite addTabPage(TabFolder folder, String label) {
		TabItem tab = new TabItem(folder, SWT.NONE);
		tab.setText(label);
		Composite page = new Composite(folder, SWT.NONE);
		page.setLayout(new FillLayout());
		tab.setControl(page);
		return page;
	}
	
    protected final class TeeFeature {
        private String name;
        private String classpath;
        private String issues;
        private boolean visible;
        private boolean leaf;

        public TeeFeature(String name, String classpath, String issues,
                              boolean visible, boolean leaf) {
            this.name = name;
            this.classpath = classpath;
            this.issues = issues;
            this.visible = visible;
        }

        public TeeFeature(String name, String classpath) {
            this(name, classpath, "", true, true);
        }

        public String getIssues() {
            return issues;
        }

        public boolean isFinished() {
            return ((issues.length() == 0) || issues.equals("0"));
        }

        public boolean isLeaf() {
            return leaf;
        }

        public boolean isSample() {
            return !this.classpath.equalsIgnoreCase("");
        }

        public boolean isVisible() {
            return visible;
        }

        public String toString() {
            return name;
        }
    }
    
    public TeeFeature createFeatureNodeOf(Node childElement) {
        NamedNodeMap elementAttributes = childElement.getAttributes();

        TeeFeature tmpNode = null;

        if (elementAttributes != null &&
            elementAttributes.getLength() > 0) {

            boolean hasChilds = (childElement.getChildNodes().getLength() > 0);

            Node attribute = elementAttributes.getNamedItem("caption");
            String tmpCaption = (attribute != null) ?
                                attribute.getNodeValue() :
                                "";
            attribute = elementAttributes.getNamedItem("classpath");
            String tmpClassPath = (attribute != null) ?
                                  attribute.getNodeValue() :
                                  "";
            attribute = elementAttributes.getNamedItem("visible");
            boolean tmpVisible = (attribute != null) ?
                                 attribute.getNodeValue().equals("1") : true;
            attribute = elementAttributes.getNamedItem("issues");
            String tmpIssues = (attribute != null) ? attribute.getNodeValue() :
                               "";

            if (!fullTree) {
                if (tmpIssues.equals("") || tmpIssues.equals("0")) {
                } else {
                    tmpVisible = hasChilds;
                    if (hasChilds) {
                        tmpClassPath = "";
                    };
                    tmpIssues = "";
                }
            } else {
                tmpVisible = true;
            }

            tmpNode = new TeeFeature(
                    tmpCaption,
                    tmpClassPath,
                    tmpIssues,
                    tmpVisible,
                    !hasChilds);
        }
        return tmpNode;
    }	
    
	private void addChildren(Tree tree, TreeItem parentItem,
            Node parentXMLElement) {

		NodeList childElements = parentXMLElement.getChildNodes();

		for (int i = 0; i < childElements.getLength(); i++) {
			Node childElement = childElements.item(i);

			if (!(childElement instanceof Text ||
					childElement instanceof Comment)) {

				boolean hasChilds = (childElement.getChildNodes().getLength() > 0);

				TeeFeature feature = createFeatureNodeOf(childElement);

				if (feature.isVisible()
						|| Parameters.getShowAllFeatures()
						|| hasChilds) {

					if (feature.isSample()) {
						treeNodeCount++;
					};

					TreeItem childItem;
					if (parentItem==null) { 
						childItem = new TreeItem(tree, SWT.NONE);
					} else {
						childItem = new TreeItem(parentItem, SWT.NONE);
					}
					childItem.setText(feature.toString());
					childItem.setData(feature);
					
					addChildren(tree, childItem, childElement);
				}
			}
		}
	}

	private int treeNodeCount = 0;		
	
    protected Tree createFeaturesTree(Composite parent, String section) {
    	Tree allTree = new Tree(parent, SWT.SINGLE);    	    	        	
    	
        try {
            DocumentBuilderFactory builderFactory =
                    DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = builderFactory.newDocumentBuilder();

            Document document = builder.parse(getClass().getResource(FEATURES_URL).toString()); //TeeChartDemo.class                                            

            document.getDocumentElement().normalize();
            Element rootElement = document.getDocumentElement();

            NodeList tmpList = document.getElementsByTagName(section);
            
            treeNodeCount = 0;

            if (tmpList != null) {
                rootElement = (Element) tmpList.item(0);
                addChildren(allTree, null, rootElement);
            }

        } catch (Exception e) {
            String errorMessage = "Error creating feature tree: " + e;
            System.err.println(errorMessage);
            e.printStackTrace();
        }
        return allTree;
    }
	
    private Composite createNavigationUI(Composite parent) {
    	final Color bgColor = new Color(display, 234, 238, 255);
    	TabFolder folder = new TabFolder(parent, SWT.NONE);   	
    	folder.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {				
				bgColor.dispose();
			}
    	});    	    	
    	
    	SelectionAdapter featureSelection = new SelectionAdapter()
    	{
    		public void widgetSelected(SelectionEvent e)
    		{
    			//System.out.println("Selection:"+ e.item.toString());
    			if (e.item.getData() != null) {
    				showSamplePane((TeeFeature)e.item.getData());
    			}
    		}
    	};    			
    	
    	/* New Features */
    	
    	/* All Features */
    	Composite allPage = addTabPage(folder, "All Features");
    	Tree allTree = createFeaturesTree(allPage, "all");
    	allTree.setBackground(bgColor);
    	allTree.addSelectionListener(featureSelection); 
    	
    	activeTree = allTree; //only one tree at the moment
    	
    	return folder;
    }
    
    private Composite createTopUI(Composite parent) {
    	Composite pane = new Composite(parent, SWT.NONE);
    	final Color bgColor = new Color(display, 241, 243, 243);
    	final Image tcImage = new Image(display, getClass().getResourceAsStream(IMAGE_TEECHARTHEADER_URL));
    	final Image logoImage = new Image(display, getClass().getResourceAsStream(IMAGE_TEECHARTLOGO_URL));
    	final Font font = new org.eclipse.swt.graphics.Font(display, "Arial", 14, SWT.BOLD);

    	GridLayout gridLayout = new GridLayout();
    	gridLayout.numColumns = 10;    	
    	pane.setLayout(gridLayout);
    	pane.setBackground(bgColor);

    	final Navigate navigateListener = new Navigate();
    	leftButton = WidgetFactory.createPushButton(
    			pane,
    			"<",
    			"Previous",
    			navigateListener
    	);        
    	rightButton = WidgetFactory.createPushButton(
    			pane, 
    			">",
    			"Next", 
    			navigateListener
    	);                   

    	pane.setBackgroundImage(tcImage);

    	shell.setBackgroundMode(SWT.INHERIT_DEFAULT);
      
    	Label spacer1 = new Label(pane, SWT.NONE);
    	spacer1.setLayoutData(new GridData(SWT.TRANSPARENCY_ALPHA, SWT.TOP, true, false));
    	Label spacer2 = new Label(pane, SWT.NONE);
    	spacer2.setLayoutData(new GridData(SWT.TRANSPARENCY_ALPHA, SWT.TOP, true, false));
    	{
    		Label spacer3 = new Label(pane, SWT.LEFT);
    		spacer3.setForeground(new Color(display,225,225,225));
    	spacer3.setFont(font);
    	spacer3.setText("SWT Examples");
    	}
    	
    	Label spacer4 = new Label(pane, SWT.NONE);
    	spacer4.setLayoutData(new GridData(SWT.TRANSPARENCY_ALPHA, SWT.TOP, true, false));

    	Label spacer4b = new Label(pane, SWT.NONE);
    	spacer4b.setLayoutData(new GridData(SWT.TRANSPARENCY_ALPHA, SWT.TOP, true, false));

    	Label lblHeader = new Label(pane, SWT.CENTER);
    	if (logoImage != null) {
    		lblHeader.setImage(logoImage);
    		lblHeader.setBounds(logoImage.getBounds());
    	}
    	lblHeader.setLayoutData(new GridData(SWT.TRANSPARENCY_ALPHA, SWT.NONE, true, false));

    	WidgetFactory.createPushButton(
    			pane, 
    			"About...",
    			"About TeeChart for Java",
    			new About()
    	);

    	WidgetFactory.createPushButton(
    			pane, 
    			"Close", 
    			"Close TeeChart Demo Application",
    			new Close()
    	);            	

    	pane.addDisposeListener(new DisposeListener() {
    		public void widgetDisposed(DisposeEvent e) {
    			tcImage.dispose();
    			logoImage.dispose();
    			bgColor.dispose();
    			font.dispose();
    		}
    	});

    	return pane;
    }
    
    private void initUI() {
    	GridLayout gl = new GridLayout();
    	gl.numColumns = 1;    	
    	shell.setLayout(gl);
    	Composite topPane = createTopUI(shell);
    	topPane.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
    	    	
    	SashForm sashForm = new SashForm(shell, SWT.HORIZONTAL);
    	sashForm.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));    	    	
    	
    	createNavigationUI(sashForm);    	
    	sampleContainer = new Composite(sashForm, SWT.NONE);
    	sampleContainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
    	
    	sashForm.setWeights(new int[] {30,70});
    	
    	Composite bottomPane = createBottomUI(shell);
    	bottomPane.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
    }
    
    private void disposeComposite(Composite c) {
    	while (c.getChildren().length > 0) {
    		if (c.getChildren()[0] instanceof Composite) {
    			disposeComposite(((Composite)c.getChildren()[0]));
    		} else {	
    			while (c.getChildren().length > 0) {
    				c.getChildren()[0].dispose();
    			}
    		}
    	}
    	if (c != sampleContainer) { c.dispose(); };     	
    }
    
    private void showSamplePane(TeeFeature feature) {
    	//disposeComposite(sampleContainer);
    	while (sampleContainer.getChildren().length > 0) {
    		sampleContainer.getChildren()[0].dispose();
    	}
    	
    	if (feature.classpath.equals("")) { return; }

        try {
        	String className = Parameters.featuresName + feature.classpath;
            Class cls = Class.forName(className);
          	Constructor constructor = cls.getConstructor(new Class[]{Composite.class});
           	Composite tmp = (Composite)constructor.newInstance(new Object[]{sampleContainer});
           	tmp.setSize(sampleContainer.getSize());
           	statusLabel.setText(feature.name + "  (" + className + ".java)");
        } catch (InvocationTargetException e) {
            System.out.println(e);
            e.printStackTrace();                                
        } catch (NoSuchMethodException e) {
            System.out.println(e);
            e.printStackTrace();                                
        } catch (InstantiationException e) {
            System.out.println(e);
            e.printStackTrace();            
        } catch (IllegalAccessException e) {
            System.out.println(e);
            e.printStackTrace();           
        } catch (ClassNotFoundException e) {        	
            System.out.println(e);
            e.printStackTrace();
            statusLabel.setText("");
        }
    }    
    
    private boolean fullTree = false;
    private Tree activeTree;
       
    public static final String FEATURES_URL = "features/features.xml";
    public static final String IMAGE_TEECHARTHEADER_URL = "features/images/TeeChartHeader.jpg";
    public static final String IMAGE_TEECHARTLOGO_URL = "features/images/TeeChartLogo.gif";

	/**
	 * @param args the command line arguments
	 */
	public static void main(String[] args) {
		Parameters.loadProperties();
		TeeChartSWT instance = new TeeChartSWT();
		instance.run();
		Parameters.saveProperties();
	}
}
