/*
 * Main.java
 *
 * <p>Copyright: Copyright (c) 2004-2007 by Steema Software SL. All Rights
 * Reserved.</p>
 *
 * <p>Company: Steema Software SL</p>
 */

import com.steema.teechart.BevelStyle;
import com.steema.teechart.TChart;
import com.steema.teechart.drawing.Gradient;
import com.steema.teechart.drawing.GradientDirection;
import com.steema.teechart.editors.AboutDialog;
import com.steema.teechart.tools.Annotation;
import com.steema.teechart.drawing.Color;
import com.steema.teechart.drawing.ChartFont;
import com.steema.teechart.editors.Utils;

import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import features.utils.WaitToolkit;
import org.w3c.dom.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import features.Parameters;
import java.awt.Component;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.border.BevelBorder;
import java.awt.event.ActionListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultTreeModel;

import javax.swing.plaf.metal.MetalLookAndFeel;
import com.sun.java.swing.plaf.motif.MotifLookAndFeel;
import java.awt.Image;

//import com.sun.java.swing.plaf.gtk.*;

/**
 *
 * @author tom
 */
final public class TeeChartDemo extends JPanel implements ActionListener {

    JMenuItem menuTest;
    JMenuItem menuShowAll;
    JMenuItem menuLookDefault;
    JMenuItem menuLookMetal;
    JMenuItem menuLookMotif;
    JMenuItem menuLookGTK;

    public JTree tmpTree;

    /** Creates a new instance of Main */
    public TeeChartDemo() {
        super(new BorderLayout());

        initComponents();

        tmpTree = new JTree(buildFeaturesTree());

        JComponent featuresTab = buildFeaturesTab(tmpTree);

        splitPane = new JSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                featuresTab,
                samplePane
                    );

        splitPane.setBorder(BorderFactory.createEmptyBorder());
        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(200);
        splitPane.setDividerSize(7);

        //Provide minimum sizes for the two components in the split pane.
        Dimension minimumSize = new Dimension(100, 50);
        featuresTab.setMinimumSize(minimumSize);
        samplePane.setMinimumSize(minimumSize);

        //Provide a preferred size for the split pane.
        splitPane.setPreferredSize(new Dimension(400, 200));

        add(buildHeaderPane(), BorderLayout.PAGE_START);
        add(splitPane, BorderLayout.CENTER);

        statusPanel.setPreferredSize(new Dimension(0, 24));
        statusPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.
                LOWERED));

        statusPanel.add(statusLabel);
        add(statusPanel, BorderLayout.SOUTH);

        addPopupMenu();

        showBasicDemo(tmpTree);
    }

    private JMenuItem addMenu(JPopupMenu popupMenu, String text) {
        JMenuItem result = new JMenuItem(text);
        result.addActionListener(this);
        popupMenu.add(result);
        return result;
    }

    private void addPopupMenu() {
        final JPopupMenu popupMenu = new JPopupMenu("Menu");

        menuTest = addMenu(popupMenu, "Test all demos (" +
                                 Integer.toString(treeNodeCount) + ")");

        menuShowAll = new JCheckBoxMenuItem("Show all tree nodes");
        menuShowAll.addActionListener(this);
        popupMenu.add(menuShowAll);
        popupMenu.add(new JSeparator());

        menuLookDefault = addMenu(popupMenu, "Default Look");
        menuLookMetal = addMenu(popupMenu, "Metal Look");
        menuLookMotif = addMenu(popupMenu, "Motif Look");
        menuLookGTK = addMenu(popupMenu, "GTK Look");

        statusPanel.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent evt) {
                if (evt.isPopupTrigger()) {
                    popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
                }
            }

            public void mouseReleased(MouseEvent evt) {
                if (evt.isPopupTrigger()) {
                    popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
                }
            }
        });
    }

    public void actionPerformed(ActionEvent e) {
        String tmp = e.getActionCommand();

        if (tmp.equals(menuTest.getText())) {
            walk(tmpTree.getModel(), tmpTree.getModel().getRoot());
        } else
        if (tmp.equals(menuShowAll.getText())) {
            fullTree = menuShowAll.isSelected();

            tmpTree.setModel(new DefaultTreeModel(buildFeaturesTree(), false));
            tmpTree.setCellRenderer(new TeeFeatureNodeRenderer(fullTree));

            tmpTree.expandRow(0);
        } else
        if (tmp.equals(menuLookDefault.getText())) {
            setLook(UIManager.getSystemLookAndFeelClassName());
        } else
        if (tmp.equals(menuLookMetal.getText())) {
            setLook("javax.swing.plaf.metal.MetalLookAndFeel");
        } else
        if (tmp.equals(menuLookMotif.getText())) {
            setLook("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
        } else
        if (tmp.equals(menuLookGTK.getText())) {
            setLook("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
        }
    }

    private void setLook(String className) {
        try {
            UIManager.setLookAndFeel(className);
            SwingUtilities.updateComponentTreeUI(window);
            window.pack();

        } catch (UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
    }

    protected void walk(TreeModel model, Object o) {
        int count = model.getChildCount(o);

        for (int t = 0; t < count; t++) {
            Object child = model.getChild(o, t);

            if (model.isLeaf(child)) {
                final TeeFeatureNode feature = (TeeFeatureNode) ((
                        DefaultMutableTreeNode) child).getUserObject();

                showSamplePane(feature);
                paintAll(getGraphics());

            } else {
                walk(model, child);
            }
        }
    }

    final public class AboutAction extends AbstractAction {
        public AboutAction(String text, ImageIcon icon,
                           String desc, Integer mnemonic) {
            super(text, icon);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }

        public void actionPerformed(ActionEvent e) {
            AboutDialog dialog = new AboutDialog(window);
            dialog.setVisible(true);
        }
    }


    final public class CloseAction extends AbstractAction {
        public CloseAction(String text, ImageIcon icon,
                           String desc, Integer mnemonic) {
            super(text, icon);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }

        public void actionPerformed(ActionEvent e) {
            Parameters.saveProperties();
            window.dispose();
        }
    }


    public class LeftAction extends AbstractAction {
        public LeftAction(String text, ImageIcon icon,
                          String desc, Integer mnemonic) {
            super(text, icon);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }

        public void actionPerformed(ActionEvent e) {
            int tmpRowSelection = tmpTree.getLeadSelectionRow()-1;

            if (tmpRowSelection >= 0)
                tmpTree.setSelectionRow(tmpRowSelection);
            else
              tmpTree.setSelectionRow(0);

            tmpTree.expandRow(tmpRowSelection);
        }
    }


    public class RightAction extends AbstractAction {
        public RightAction(String text, ImageIcon icon,
                           String desc, Integer mnemonic) {
            super(text, icon);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }

        public void actionPerformed(ActionEvent e) {
            int tmpRowSelection = tmpTree.getLeadSelectionRow()+1;
            if (tmpTree.getPathForRow(tmpRowSelection)!=null) {
                tmpTree.setSelectionRow(tmpRowSelection);
                tmpTree.expandRow(tmpRowSelection);
            }
            else
                tmpRowSelection -= 1;
        }
    }

    private static JFrame window;
    private JSplitPane splitPane;
    JButton closeButton, aboutButton, leftButton, rightButton;
    private Action aboutAction, closeAction, leftAction, rightAction;
    private JPanel samplePane;
    private static JPanel defaultPanel = new JPanel();
    private JPanel statusPanel = new JPanel();
    private JLabel statusLabel = new JLabel();

    private boolean showWait = false;
    private boolean fullTree = false;

    /**
     * Creates, intializes and configures the UI components.
     */
    private void initComponents() {
    	{
	    	this.setPreferredSize(new java.awt.Dimension(806, 284));
    	}
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setPreferredSize(new Dimension(dim.width - (dim.width / 3),
                                       dim.height - (dim.height / 3)));

        //Create actions
        aboutAction = new AboutAction("About...", null,
                                      "About TeeChart for Java", null);
        closeAction = new CloseAction("Close", null,
                                      "Close TeeChart Demo Application", null);
        leftAction = new LeftAction("<", null,
                                      "Previous", null);
        rightAction = new RightAction(">", null,
                                      "Next", null);

        //create buttons
        closeButton = new JButton(closeAction);
        aboutButton = new JButton(aboutAction);

        // navigate buttons
        leftButton = new JButton(leftAction);
        rightButton = new JButton(rightAction);

        samplePane = buildSamplePane();
    }

    protected final class TeeFeatureNode {
        private String name;
        private String classpath;
        private String issues;
        private boolean visible;
        private boolean leaf;

        public TeeFeatureNode(String name, String classpath, String issues,
                              boolean visible, boolean leaf) {
            this.name = name;
            this.classpath = classpath;
            this.issues = issues;
            this.visible = visible;
        }

        public TeeFeatureNode(String name, String classpath) {
            this(name, classpath, "", true, true);
        }

        public String getIssues() {
            return issues;
        }

        public boolean isFinished() {
            return ((issues.length() == 0) || issues.equals("0"));
        }

        public boolean isLeaf() {
            return leaf;
        }

        public boolean isSample() {
            return !this.classpath.equalsIgnoreCase("");
        }

        public boolean isVisible() {
            return visible;
        }

        public String toString() {
            return name;
        }
    }


    final protected static class TeeFeatureNodeRenderer extends
            DefaultTreeCellRenderer {

        private boolean fullTree = true;

        public TeeFeatureNodeRenderer(boolean fullTree) {
            super();
            this.fullTree = fullTree;
            setBackgroundNonSelectionColor(FEATURES_TREECOLOR);

            ImageIcon tmp = new ImageIcon(TeeFeatureNodeRenderer.class.
                                          getResource(
                                                  "features/images/treeleaf.png"));
            ImageIcon tmp2 = new ImageIcon(Utils.makeTransparent(tmp.getImage(),
                    new Color(234, 238, 255)));

            setLeafIcon(tmp2);
        }

        public Component getTreeCellRendererComponent(
                JTree tree,
                Object value,
                boolean selected,
                boolean expanded,
                boolean leaf,
                int row,
                boolean hasFocus) {

            Component c = super.getTreeCellRendererComponent(
                    tree, value, selected,
                    expanded, leaf, row,
                    hasFocus);

            if (fullTree && (value != null)) {
                Object tmpObject = ((DefaultMutableTreeNode) value).
                                   getUserObject();
                if (tmpObject != null) {
                    TeeFeatureNode tmpNode = (TeeFeatureNode) tmpObject;
                    if (!tmpNode.isVisible()) {
                        c.setForeground(Color.GRAY);
                        setToolTipText(null);
                    } else if (!tmpNode.isFinished()) {
                        c.setForeground(Color.MAGENTA);
                        setToolTipText(tmpNode.getIssues());
                    } else {
                        setToolTipText(null);
                    }
                }
            }

            return c;
        }
    }


    protected JComponent buildHeaderPane() {

    	JPanel spacerPane = new JPanel();
    	{
    		spacerPane.setLayout(new FlowLayout(FlowLayout.CENTER));
    		spacerPane.setOpaque(false);
    	}

    	JLabel headerLabel = new JLabel();
    	{
    		headerLabel.setText("Swing Examples");
    		headerLabel.setForeground(Color.fromArgb(255,255,255));
    		headerLabel.setFont(new java.awt.Font("Default", java.awt.Font.PLAIN, 18));
    	}

    	JPanel headerPanel = new JPanel();
    	{
    		headerPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    		headerPanel.add(leftButton);
    		headerPanel.add(rightButton);
    		headerPanel.add(headerLabel, BorderLayout.LINE_END);
    		headerPanel.setOpaque(false);
    	}

    	ImagePanel logoPanel = new ImagePanel(new ImageIcon(TeeChartDemo.class.getResource("features/images/TeeChartLogo.gif")).getImage());
    	{
    		logoPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    	}        

    	JPanel buttonPane = new JPanel();
    	{

    		buttonPane.setLayout(new FlowLayout(FlowLayout.CENTER));
    		buttonPane.setBackground(Color.fromArgb(247,189,0));
    		buttonPane.add(logoPanel, BorderLayout.LINE_END);
    		buttonPane.add(spacerPane, BorderLayout.CENTER);
    		buttonPane.add(aboutButton);
    		buttonPane.add(closeButton);
    		buttonPane.setOpaque(false);
    	}


    	ImagePanel headerPane = new ImagePanel(new ImageIcon(TeeChartDemo.class.getResource("features/images/TeeChartHeader.jpg")).getImage(),true);
    	{
    		headerPane.setBackground(Color.fromArgb(45,140,208));
    		headerPane.setLayout(new BorderLayout());
    		headerPane.add(headerPanel, BorderLayout.LINE_START);

    		headerPane.add(buttonPane, BorderLayout.LINE_END);
    	}

    	return headerPane;
    }

    // Create an Annotation tool
    private Annotation getBanner() {
        Annotation a = new Annotation("TeeChart Pro for Java");
        a.setLeft(60);
        a.setTop( -5);
        a.getShape().getFont().setName("Arial Black");

        Gradient ag = a.getShape().getGradient();
        ag.setVisible(true);
        ag.setDirection(GradientDirection.HORIZONTAL);
        ag.setStartColor(Color.darkGray);

        ChartFont f = a.getShape().getFont();
        f.setSize(26);
        f.setItalic(true);

        f.getOutline().setColor(Color.WHITE);
        f.getOutline().setVisible(true);

        f.getShadow().setVisible(true);
        f.getShadow().setHorizSize(2);
        f.getShadow().setVertSize(2);

        Gradient g = f.getGradient();
        g.setVisible(true);
        g.setStartColor(Color.GREEN);
        g.setMiddleColor(Color.NAVY);
        g.setEndColor(Color.red);
        g.setDirection(GradientDirection.HORIZONTAL);

        a.getShape().setTransparent(true);

        return a;
    }

    public TeeFeatureNode getFeatureNode(TreePath path) {

        if (path != null) {
            Object comp = path.getLastPathComponent();
            DefaultMutableTreeNode tmpNode = (DefaultMutableTreeNode) comp;

            if (tmpNode != null) {
                Object tmpObject = tmpNode.getUserObject();

                if (tmpObject != null) {
                    return (TeeFeatureNode) tmpObject;
                }
            }
        }

        return null;
    }

    public TeeFeatureNode createFeatureNodeOf(Node childElement) {
        NamedNodeMap elementAttributes = childElement.getAttributes();

        //String treeNodeLabel = ""; // = childElement.getNodeName();

        TeeFeatureNode tmpNode = null;

        if (elementAttributes != null &&
            elementAttributes.getLength() > 0) {

            boolean hasChilds = (childElement.getChildNodes().getLength() > 0);

            Node attribute = elementAttributes.getNamedItem("caption");
            String tmpCaption = (attribute != null) ?
                                attribute.getNodeValue() :
                                "";
            attribute = elementAttributes.getNamedItem("classpath");
            String tmpClassPath = (attribute != null) ?
                                  attribute.getNodeValue() :
                                  "";
            attribute = elementAttributes.getNamedItem("visible");
            boolean tmpVisible = (attribute != null) ?
                                 attribute.getNodeValue().equals("1") : true;
            attribute = elementAttributes.getNamedItem("issues");
            String tmpIssues = (attribute != null) ? attribute.getNodeValue() :
                               "";

            if (!fullTree) {
                if (tmpIssues.equals("") || tmpIssues.equals("0")) {
                } else {
                    tmpVisible = hasChilds;
                    if (hasChilds) {
                        tmpClassPath = "";
                    };
                    tmpIssues = "";
                }
            } else {
                tmpVisible = true;
            }

            tmpNode = new TeeFeatureNode(
                    tmpCaption,
                    tmpClassPath,
                    tmpIssues,
                    tmpVisible,
                    !hasChilds);
        }
        return tmpNode;
    }

    private void addChildren(DefaultMutableTreeNode parentTreeNode,
                             Node parentXMLElement) {

        NodeList childElements = parentXMLElement.getChildNodes();

        for (int i = 0; i < childElements.getLength(); i++) {
            Node childElement = childElements.item(i);

            if (!(childElement instanceof Text ||
                  childElement instanceof Comment)) {

                boolean hasChilds = (childElement.getChildNodes().getLength() > 0);

                TeeFeatureNode tmpInfo = createFeatureNodeOf(childElement);

                if (tmpInfo.isVisible()
                    || Parameters.getShowAllFeatures()
                    || hasChilds) {

                    if (tmpInfo.isSample()) {
                        treeNodeCount++;
                    };

                    DefaultMutableTreeNode childTreeNode =
                            new DefaultMutableTreeNode(tmpInfo);
                    parentTreeNode.add(childTreeNode);
                    addChildren(childTreeNode, childElement);
                }
            }
        }
    }

    private int treeNodeCount = 0;

    protected DefaultMutableTreeNode buildFeaturesTree() {
        try {
            DocumentBuilderFactory builderFactory =
                    DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = builderFactory.newDocumentBuilder();

            Document document = builder.parse(TeeChartDemo.class.
                                              getResource(FEATURES_URL)
//                    ClassLoader.getSystemResource(FEATURES_URL)
                                              .toString());

            document.getDocumentElement().normalize();
            Element rootElement = document.getDocumentElement();
            DefaultMutableTreeNode rootTreeNode = null;
            NodeList tmpList = document.getElementsByTagName("all");

            treeNodeCount = 0;

            if (tmpList != null) {
                rootElement = (Element) tmpList.item(0);
                rootTreeNode = new DefaultMutableTreeNode(
                        createFeatureNodeOf(rootElement));

                addChildren(rootTreeNode, rootElement);
            }

            return rootTreeNode;

        } catch (Exception e) {
            String errorMessage = "Error creating feature tree: " + e;
            System.err.println(errorMessage);
            e.printStackTrace();
            return (new DefaultMutableTreeNode(errorMessage));
        }
    }

    public JComponent buildFeaturesTab(JTree tmpTree) {
        JTabbedPane tmpPane = new JTabbedPane();
        tmpPane.setBorder(BorderFactory.createEmptyBorder());

        tmpTree.setBackground(FEATURES_TREECOLOR);

        tmpTree.setCellRenderer(new TeeFeatureNodeRenderer(fullTree));

        tmpTree.setEditable(false);
        tmpTree.setRootVisible(false);
        tmpTree.expandRow(0);

        if (showWait) {

            tmpTree.addMouseListener(new MouseAdapter() {

                public TeeFeatureNode getNode(MouseEvent evt) {

                    if (evt.getSource() instanceof JTree) {
                        JTree t = (JTree) evt.getSource();
                        TreePath path = t.getPathForLocation(evt.getX(),
                                evt.getY());

                        return getFeatureNode(path);

                    } else {
                        return null;
                    }
                };

                public void mousePressed(MouseEvent evt) {
                    TeeFeatureNode sampleNode = getNode(evt);

                    if (sampleNode != null) {
                        if (sampleNode.isSample()) {

                            //REMOVED. To avoid big flicker.
                            //splitPane.setRightComponent(defaultPanel);

                            //Slow!
                            //WaitToolkit.startWait(window.getRootPane());
                        }
                    }
                };

                public void mouseReleased(MouseEvent evt) {
                    try {
                        TeeFeatureNode sampleNode = getNode(evt);

                        if (sampleNode != null) {
                            showSamplePane(sampleNode);
                        }

                    } catch (Exception ex) {
                        ex.printStackTrace();

                        WaitToolkit.stopWait(window.getRootPane());
                    }
                }
            });

        } else {
            tmpTree.addTreeSelectionListener(new TreeSelectionAdapter() {
                public void valueChanged(TreeSelectionEvent e) {
                    TeeFeatureNode sampleNode = getFeatureNode(e.getPath());

                    if (sampleNode != null) {
                        showSamplePane(sampleNode);
                    }
                }
            });
        }

        ToolTipManager.sharedInstance().registerComponent(tmpTree);

        /* this is the first version, so everything is new :-)
                 tmpPane.addTab(
                "What's New?",
                new JScrollPane(tmpTree)
                );
         */
        tmpPane.addTab("All features", new JScrollPane(tmpTree));

        return tmpPane;
    }

    private void showBasicDemo(JTree tmpTree) {
        tmpTree.setSelectionRow(1);
    }

    protected static JPanel buildSamplePane() {
        JPanel tmpPane = new JPanel();
        tmpPane.setBorder(BorderFactory.createEmptyBorder());
        return tmpPane;
    }

    private void showSamplePane(TeeFeatureNode sampleInfo) {
        if (sampleInfo.classpath.equals("")) {
            return;
        }

        try {
            String className = Parameters.featuresName + sampleInfo.classpath;

            Class cls = Class.forName(className);
            JPanel tmpPanel = (JPanel) cls.newInstance();

            if (showWait) {
                tmpPanel.addComponentListener(new ComponentAdapter() {
                    public void componentResized(ComponentEvent e) {
                        WaitToolkit.stopWait(window.getRootPane());
                    }
                });
            }

            int oldPos = splitPane.getDividerLocation();
            splitPane.setRightComponent(tmpPanel);
            splitPane.setDividerLocation(oldPos);

            statusLabel.setText(sampleInfo.name + "  (" + className + ".java)");

        } catch (InstantiationException e) {
            System.out.println(e);
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            System.out.println(e);
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            System.out.println(e);
            e.printStackTrace();
        }
    }

    private static void createAndShowGUI() {
        JFrame.setDefaultLookAndFeelDecorated(true);
        window = new JFrame("TeeChart for Java");
        window.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        window.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                Parameters.saveProperties();
                System.exit(0);
            }
        });

        window.setContentPane(new TeeChartDemo());
        window.pack();

        window.setLocationRelativeTo(null);
        window.setVisible(true);
    }

    public static final Color FEATURES_TREECOLOR = new Color(234, 238, 255);
    public static final String FEATURES_URL = "/features/features.xml";

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Parameters.loadProperties();

        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    UIManager.setLookAndFeel(
                            UIManager.getSystemLookAndFeelClassName());
                    createAndShowGUI();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        });
    }
}

final class ImagePanel extends JPanel {

  private Image img;
  private int imgWidth;
  private int imgHeight;
  private boolean resizeImg = false;

  public ImagePanel(Image img) {
  	this(img,false);
  }
  
  public ImagePanel(String img) {
    this(new ImageIcon(img).getImage(),false);
  }
 
  public ImagePanel(String img, boolean resize) {
    this(new ImageIcon(img).getImage(),false);
  }

  public ImagePanel(Image img, boolean resize) {
    this.img = img;
    resizeImg = resize; 
    
    setBackground(Color.fromArgb(45,140,208));
    setOpaque(true);
    
    imgWidth=img.getWidth(null);
    imgHeight=img.getHeight(null);
    Dimension size = new Dimension(img.getWidth(null), img.getHeight(null));
    setPreferredSize(size);
    setMinimumSize(size);
    setMaximumSize(size);
    setSize(size);
    setLayout(null);
  }

  public void paintComponent(java.awt.Graphics g) {
  	if (resizeImg)
  	{
      if (imgWidth<getWidth()) imgWidth=getWidth();
      if (imgHeight<getHeight()) imgHeight=getHeight();
  	}
    g.drawImage(img, 0, 0, imgWidth, imgHeight, null);
  }

}

abstract class TreeSelectionAdapter implements TreeSelectionListener {
    public abstract void valueChanged(TreeSelectionEvent evt);
}

