/*
 * DownSamplingDemo.java
 *
 * <p>Copyright: Copyright (c) 2005-2008 by Steema Software SL. All Rights
 * Reserved.</p>
 *
 * <p>Company: Steema Software SL</p>
 */

package features.function;

import com.steema.teechart.drawing.Color;
import com.steema.teechart.functions.DownSampling;
import com.steema.teechart.functions.DownSamplingMethod;
import com.steema.teechart.styles.Points;
import com.steema.teechart.styles.FastLine;

import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


import features.ChartSamplePanel;

public class DownSamplingDemo extends ChartSamplePanel
    implements ActionListener, ChangeListener {

    private Points pointS;
    private FastLine fastL;
    private DownSampling sFunction;
    
    private double[] xValues, yValues;

    
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == DownSampleComboBox) {
            DownSamplingMethod dMethod;
            
            switch (DownSampleComboBox.getSelectedIndex())
            {
              case 0:  dMethod = DownSamplingMethod.MAX; break;
              case 1:  dMethod = DownSamplingMethod.MIN; break;
              case 2:  dMethod = DownSamplingMethod.MINMAX; break;
              case 3:  dMethod = DownSamplingMethod.MINMAXFIRSTLAST; break;
              case 4:  dMethod = DownSamplingMethod.MINMAXFIRSTLASTNULL; break;
              case 5:  dMethod = DownSamplingMethod.AVERAGE; break;
              default: dMethod = DownSamplingMethod.MAX; break;
            }
           
            sFunction.setMethod(dMethod);
            chart1.getSeries(1).clear();
            chart1.getSeries(1).checkDataSource();
            //chart1.getSeries(1).invalidate();
        }
    }
    
    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();
        if (source instanceof JSpinner) {
            SpinnerModel tmpModel = ((JSpinner)source).getModel();
            if (tmpModel instanceof SpinnerNumberModel) {
                int dCount = ((SpinnerNumberModel)tmpModel).getNumber().intValue();
                if ((dCount > -1) && (dCount < 2001)) {
                    if (source == displaySpinner) {
                        sFunction.setDisplayedPointCount(dCount);
                        chart1.getSeries(1).checkDataSource();
                    }
                }
            }
        }
    }
    
    /**
     * Creates a new instance of CCIDemo
     */
    public DownSamplingDemo() {
        super();
        DownSampleComboBox.addActionListener(this);
        displaySpinner.addChangeListener(this);
    }

    public void ChartInit()
    {
        createArrays();

        pointS = new com.steema.teechart.styles.Points(chart1.getChart());
        fastL = new com.steema.teechart.styles.FastLine(chart1.getChart());
        
        pointS.getPointer().setHorizSize(0);
        pointS.getPointer().setVertSize(0);
        pointS.getPointer().getPen().setVisible(false);
        
        //chart1.getSeries(0).setActive(false);

        chart1.getSeries(0).add(xValues, yValues);
        
        sFunction = new com.steema.teechart.functions.DownSampling(chart1.getChart());

        fastL.setTitle("DownSample");
        fastL.setColor(Color.GREEN);
        
        chart1.getSeries(1).setDataSource(pointS);
        chart1.getSeries(1).setFunction(sFunction);
//        chart1.getSeries(1).setActive(false);
        
    }
    
    protected void initComponents() {
        super.initComponents();

        ChartInit();
        sFunction.setMethod(DownSamplingMethod.MINMAXFIRSTLASTNULL);
        sFunction.setDisplayedPointCount(1000);
        chart1.getSeries(1).checkDataSource();

        DownSampleComboBox = new JComboBox(DownSampleMethodStrings);
        DownSampleComboBox.setSelectedIndex(4);
        
        chart1.addChartMotionListener(new com.steema.teechart.events.ChartMotionListener() {
          public void scrolled(com.steema.teechart.events.ChartEvent evt) {
          }
          public void zoomed(com.steema.teechart.events.ChartEvent evt) {
            onZoom(evt);
          }
          public void unzoomed(com.steema.teechart.events.ChartEvent evt) {
            unZoomed(evt);
          }
        });
        
        SpinnerModel tmpModel;
        tmpModel = new SpinnerNumberModel(
                1000,
                0,
                2000,
                100);
        displaySpinner = new JSpinner(tmpModel);
    }

    protected void initGUI() {
        super.initGUI();
        chart1.getHeader().setVisible(true);
        chart1.setText("DownSample Function Example");
        chart1.getAspect().setView3D(false);

        JLabel tmpLabel;
        JPanel tmpPane = getButtonPane();
        {
            tmpLabel = new JLabel("Style:");
            tmpLabel.setDisplayedMnemonic('S');
            tmpLabel.setLabelFor(DownSampleComboBox);
            tmpPane.add(tmpLabel);
            tmpPane.add(DownSampleComboBox);
            tmpPane.add(Box.createHorizontalStrut(10));
            tmpLabel = new JLabel("Display Count:");
            tmpLabel.setDisplayedMnemonic('D');
            tmpLabel.setLabelFor(displaySpinner);
            tmpPane.add(tmpLabel);
            tmpPane.add(displaySpinner);
            tmpPane.add(Box.createHorizontalGlue());
        }        
    }
    
    private void createArrays()
    {
      int length = 2000;

      xValues = new double[length*2];
      yValues = new double[length*2];

      for (int i = 0; i < length; i++)
      {
        xValues[i] = i;
        if (i % 20 == 0)
        {
          yValues[i] = 0.0;
        }
        else
        {
          yValues[i] = Math.round(Math.random()*100);
        }
      }
      
      for (int i = 0; i < length; i++)
      {
        xValues[i+length] = i;
        if (i % 20 == 0)
        {
          yValues[i+length] = 0.0;
        }
        else
        {
          yValues[i+length] = Math.round(Math.random()*100);
        }
      }
    }

    private void unZoomed(com.steema.teechart.events.ChartEvent evt) {
       //series 0 is the original series, although you could use any value to set the maximum
      chart1.getAxes().getBottom().setMinMax(0, chart1.getSeries(0).getXValues().getMaximum());
      chart1.getSeries(1).checkDataSource();
    }
    
    private void onZoom(com.steema.teechart.events.ChartEvent evt) {
      chart1.getSeries(1).checkDataSource();
    }
    
    private JComboBox DownSampleComboBox;
    private JSpinner displaySpinner;
    private static final String[] DownSampleMethodStrings = { "MAX", "MIN", 
        "MINMAX","MINMAXFIRSTLAST","MINMAXFIRSTLASTNULL", "AVERAGE" };

}
